
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>
#include <pspdisplay.h>
#include <psppower.h>
#include <string.h>
#include <limits.h>
#include <errno.h>
#include <pspaudiolib.h>
#include <GL/glut.h>    
#include <GL/gl.h>
#include <GL/glu.h>	
#include <stdio.h>      
#include <stdlib.h>    
#include <time.h> 

#include "glPrint.h"
#include "md2.h"
#include "texture.h"

glPrint * cadena;


int inicio_ani=1,fin_ani=39;
bool startdemo=0;
char buf[16];


GLuint texture[1];

struct Image {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
};
typedef struct Image Image;

time_t t1,t2;

int ImageLoad(char *filename, Image *image) {
    FILE *file;
    unsigned long size;                
    unsigned long i;                    
    unsigned short int planes;         
    unsigned short int bpp;             
    char temp;                          
    if ((file = fopen(filename, "rb"))==NULL)
    {
		printf("File Not Found : %s\n",filename);
		return 0;
    }
    fseek(file, 18, SEEK_CUR);

    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
	printf("Error reading width from %s.\n", filename);
	return 0;
    }
    printf("Width of %s: %lu\n", filename, image->sizeX);
    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
		printf("Error reading height from %s.\n", filename);
		return 0;
    }
    printf("Height of %s: %lu\n", filename, image->sizeY);
    
    size = image->sizeX * image->sizeY * 3;

    if ((fread(&planes, 2, 1, file)) != 1) {
		printf("Error reading planes from %s.\n", filename);
		return 0;
    }
    if (planes != 1) {
		printf("Planes from %s is not 1: %u\n", filename, planes);
		return 0;
    }

    if ((i = fread(&bpp, 2, 1, file)) != 1) {
		printf("Error reading bpp from %s.\n", filename);
		return 0;
    }
    if (bpp != 24) {
		printf("Bpp from %s is not 24: %u\n", filename, bpp);
		return 0;
    }
	
    fseek(file, 24, SEEK_CUR);

    image->data = (char *) malloc(size);
    if (image->data == NULL) {
		printf("Error allocating memory for color-corrected image data");
		return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
		printf("Error reading image data from %s.\n", filename);
		return 0;
    }

    for (i=0;i<size;i+=3) { 
		temp = image->data[i];
		image->data[i] = image->data[i+2];
		image->data[i+2] = temp;
    }
    
    return 1;
}

void LoadGLTextures() {	
    Image *image1;
    
    image1 = (Image *) malloc(sizeof(Image));
    if (image1 == NULL) {
		printf("Error allocating space for image");
		exit(0);
    }

    if (!ImageLoad("Data/neologo.bmp", image1)) {
		exit(1);
    }        

    glGenTextures(1, &texture[0]);
    glBindTexture(GL_TEXTURE_2D, texture[0]);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    glTexImage2D(GL_TEXTURE_2D, 0, 3, image1->sizeX, image1->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image1->data);
};

char Nombrejuego[20];

CMD2Model * modelo;
CMD2Model * modelo_w;

struct timeval frameStartTime, frameEndTime;

void frameStart(void) {
  gettimeofday(&frameStartTime, NULL);
}

void frameEnd(void){
  float elapsedTime; 
  gettimeofday(&frameEndTime, NULL);
  elapsedTime= frameEndTime.tv_sec - frameStartTime.tv_sec +((frameEndTime.tv_usec - frameStartTime.tv_usec)/1.0E6);
  sprintf(buf, "fps: %2.0f", 1.0/elapsedTime);
}

int window; 

float xrot, yrot, zrot;

struct Principal{
    int x;
    int y;
    int orientacion;
};
typedef struct Principal Protagonista;

Protagonista p;

void InitGL(int Width, int Height)	     
{
    LoadGLTextures();				
    glEnable(GL_TEXTURE_2D);			
    glClearColor(0,0,0,0);
    glClearDepth(1.0);				
    glDepthFunc(GL_LESS);			
    glEnable(GL_DEPTH_TEST);			
    glShadeModel(GL_SMOOTH);			
    glCullFace(GL_BACK);    
    glEnable(GL_CULL_FACE);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();				 
    gluPerspective(45.0f,(GLfloat)Width/(GLfloat)Height,0.1f,100.0f);	
    glMatrixMode(GL_MODELVIEW);
}

void ReSizeGLScene(int Width, int Height)
{
    if (Height==0)				
	    Height=1;
    glViewport(0, 0, Width, Height);		
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0f,(GLfloat)Width/(GLfloat)Height,0.1f,100.0f);
    glMatrixMode(GL_MODELVIEW);
}

void triggerHandle (int button, int state, int x, int y)
{
	if (button == GLUT_LEFT_BUTTON) {  
		if (state == GLUT_DOWN) { 
		}
		if (state == GLUT_UP) {  
		   p.orientacion--;
		   glutPostRedisplay();
		}
	}

	if (button == GLUT_RIGHT_BUTTON) {  
		if (state == GLUT_DOWN) { 
		}
		if (state == GLUT_UP) {  
		   p.orientacion++;
		   glutPostRedisplay();
        }
	}
}

void DibujaProtagonista(){
    glPushMatrix();
    glTranslatef(p.x*2,p.y*2,0);  
    glScalef(0.5,0.5,0.5);
    glRotatef(p.orientacion*90.0,0,1,0);
    glRotatef(-90,1,0,0);
    modelo->AnimateModel(inicio_ani,fin_ani,0.7);
    glPopMatrix();    
    glEnable(GL_TEXTURE_2D); 
    glColor3f(1.0,1.0,1.0);
}



void DrawGLScene()
{
	if(startdemo){
	    frameStart();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		
	    glLoadIdentity();				
	    glTranslatef(0.0f,0.0f,-5.0f);             
	    glRotatef(xrot,1.0f,0.0f,0.0f);		
	    glRotatef(yrot,0.0f,1.0f,0.0f);		
	    glRotatef(zrot,0.0f,0.0f,1.0f);		
	    glPushMatrix();
	    glScalef(0.25,0.25,0.25);
	    glScalef(0.5,0.5,0.5);
	    DibujaProtagonista();
	    glPopMatrix();
	    glPushMatrix();
	    glTranslatef(-3,0,0);
	    glScalef(0.25,0.25,0.25);
	    glScalef(0.5,0.5,0.5);
	    DibujaProtagonista();
	    glPopMatrix();
	    glPushMatrix();
	    glTranslatef(-1.5,0,0);
	    glScalef(0.25,0.25,0.25);
	    glScalef(0.5,0.5,0.5);
	    DibujaProtagonista();
	    glPopMatrix();
	    glPushMatrix();
	    glTranslatef(1.5,0,0);
	    glScalef(0.25,0.25,0.25);
	    glScalef(0.5,0.5,0.5);
	    DibujaProtagonista();
	    glPopMatrix();
	    glPushMatrix();
	    glTranslatef(3,0,0);
	    glScalef(0.25,0.25,0.25);
	    glScalef(0.5,0.5,0.5);
	    DibujaProtagonista();
	    glPopMatrix();
		frameEnd();
	    cadena->glPrintString(400,450,buf,1);
		cadena->glPrintString(30,450,Nombrejuego,1);
	}
	else{
	    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		
	    glLoadIdentity();				

	    glTranslatef(0.0f,0.0f,-5.0f);              
	    
	    glRotatef(xrot,1.0f,0.0f,0.0f);		
	    glRotatef(yrot,0.0f,1.0f,0.0f);		
	    glRotatef(zrot,0.0f,0.0f,1.0f);		

		glScalef(3.5f,2,0);
	    glBindTexture(GL_TEXTURE_2D, texture[0]);   

		glBegin(GL_QUADS);				    
		    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	
		    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	
		    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	
		    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	
		glEnd();
	    
		glBegin(GL_QUADS);				
		    glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	
		    glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	
		    glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	
		    glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	
		glEnd();					
		
		glBegin(GL_QUADS);				
		    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	
		    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	
		    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	
		    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	
		glEnd();
	    
		glBegin(GL_QUADS);				
		    glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	
		    glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	
		    glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	
		    glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	
		glEnd();					
	    
	   
		glBegin(GL_QUADS);			
		    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	
		    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	
		    glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	
		    glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	
		glEnd();					
	    
		glBegin(GL_QUADS);				
		    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	
		    glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	
		    glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	
		    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	
		glEnd();                                    

		time(&t2);
		if((t2-t1)>5){
			startdemo=1;
		}
    }
	glutSwapBuffers();
	sceDisplayWaitVblankStart ();
	
}

void keyPressed(unsigned char key, int x, int y) 
{
	switch(key){
		case 'x':			
 		     break;
       	case 'o':			
             break;
		case 'a':
			exit(0);                   
	    default:
		;
	}
}

void specialKeyPressed(int key, int x, int y) 
{

    switch (key) {    
    case GLUT_KEY_UP: 
   	break;

    case GLUT_KEY_DOWN: 
      break;

    case GLUT_KEY_LEFT: 
      break;
    
    case GLUT_KEY_RIGHT: 
      break;

    case GLUT_KEY_HOME: 
	break;
	
    default:
	break;

   }
}

void specialKeyReleased(int key, int x, int y) 
{

    switch (key) {    
    case GLUT_KEY_UP: 
 
	break;

    case GLUT_KEY_DOWN: 
 
    break;

    case GLUT_KEY_LEFT: 

    break;
    
    case GLUT_KEY_RIGHT: 

    break;

    case GLUT_KEY_HOME: 
	break;
	
    default:
	break;
    }	
}
  
  
int main(int argc, char **argv) 

{ 
    scePowerSetClockFrequency(333, 333, 166);
    char filename;

    cadena=new glPrint;
	time(&t1);
    p.x=0;
    p.y=0;
    p.orientacion=0;
    modelo = new CMD2Model;
    modelo_w = new CMD2Model;
    glutInit(&argc, argv);  
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_ALPHA | GLUT_DEPTH);  
    glutInitWindowSize(480, 272);
    glutInitWindowPosition(0, 0);  
    window = glutCreateWindow("3D ~ GDS demo program");  
    glutDisplayFunc(&DrawGLScene);  
    glutIdleFunc(&DrawGLScene);
    glutReshapeFunc(&ReSizeGLScene);
    glutKeyboardFunc(&keyPressed);
    glutSpecialFunc(&specialKeyPressed);
    glutSpecialUpFunc(&specialKeyReleased);
    glutMouseFunc(&triggerHandle);
    InitGL(480, 272);
    sprintf( buf, "fps:%d\0",60 );
    sprintf( Nombrejuego, "Demo program\0");
    cadena->BuildFont();
    cadena->scaleFont(2,2);
    modelo->Load("knight/tris.md2","knight/knight.pcx"); 
    modelo_w->Load("knight/weapon.md2","weapon/weapon.pcx"); 
    glutMainLoop();  
    return 1;
}

